/*
 * Copyright 2016 Aaron L. Zeng <a2z@mit.edu>
 */

#ifndef QUADTREE_H_
#define QUADTREE_H_

#include "./intersection_event_list.h"
#include "./line.h"

struct quadtree {
  // If `children != NULL`, then all of the segments' endpoints in the NE and NW
  // quadrants will have y-coordinates >= `y_mid`, and all of the segments'
  // endpoints in the SE and SW quadrants will have y-coordinates < `y_mid`.
  // Similarly for `x_mid`.
  vec_dimension x_mid, y_mid;

  Line **lines;
  size_t num_lines;

  // There will be either four children (`children[0]` through `children[3]`),
  // or `children` will be NULL.
  // children[0] = NW
  // children[1] = NE
  // children[2] = SW
  // children[3] = SE
  struct quadtree *children;

  double time_step;
};

typedef struct quadtree quadtree_t;

quadtree_t *quadtree_make(Line **lines, size_t n, double time_step);

// `quadtree_intersections` only increments `num_line_line_collisions`, i.e., it
// does not initialize it to zero. Intersections are appended to
// `intersection_event_list`.
void quadtree_intersections(const quadtree_t *quadtree, IntersectionEventListReducer *intersection_event_list);

void quadtree_destroy(quadtree_t *quadtree);

#endif  // QUADTREE_H_
